//
//  CaptureCardVC.swift
//
//  Created by itsol on 5/11/20.
//  Copyright © 2020 itsol. All rights reserved.
//

import UIKit
import AVFoundation

class SBKCaptureCardVC: UIViewController, AVCapturePhotoCaptureDelegate {
    @IBOutlet public weak var viewCamera: UIView!
    @IBOutlet public weak var lbDescription: UILabel!
    @IBOutlet weak var lbCopyright: UILabel!
    @IBOutlet weak var btnRotateScreen: UIButton!
    @IBOutlet weak var lbStep: UILabel!
    @IBOutlet weak var btnCapture: UIButton!
    @IBOutlet weak var imgCaution: UIImageView!
    
    var captureSession: AVCaptureSession = AVCaptureSession()
    var stillImageOutput: AVCapturePhotoOutput =  AVCapturePhotoOutput()
    var videoPreviewLayer: AVCaptureVideoPreviewLayer!
    let videoDataOutput = AVCaptureVideoDataOutput()
    
    var descriptionScreen: String = "Front of your personal card".localized()
    var checkScreen: Int = 1 // checkScreen = 1: screen front card, checkScreen = 2: screen front
    var idFront: String = ""
    var URLToken: String = ""
    var statusTakePhoto: Bool = true
    var statusValidateImage: ValidateCard = .ERROR
    var statusScreen: String = "vertical" //horizontal
    let validateInput = SBKValidateInput.shared
    
    var completionSuccessCard: ([String:Any])->Void = {_ in}
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.btnRotateScreen.layer.cornerRadius = 20
    }
    
    init() {
        super.init(nibName: "SBKCaptureCardVC", bundle: Bundle(for: SBKCaptureCardVC.self))
    }
          
    required public init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    func loadConfigUI() {
        self.lbStep.text = String(format: "Step %d/3", checkScreen)
        self.lbCopyright.textColor = UIColor.colorFromHexa(Global.colorConfig)
        self.lbDescription.textColor = UIColor.colorFromHexa(Global.colorConfig)
        if Global.frontConfig == "" {
            self.lbCopyright.font = UIFont(name: Global.frontConfig, size: 17)
            self.lbDescription.font = UIFont(name: Global.frontConfig, size: 17)
        }
        
        let label = UILabel()
        label.frame = CGRect(x: self.view.center.x - 50, y: self.viewCamera.frame.size.height / 8 + self.viewCamera.frame.origin.y - 40, width: 100, height: 20)
        label.textAlignment = .center
        label.text = self.checkScreen == 1 ? "Use front" : "Use back"
        label.textColor = UIColor.white
        
        self.view.addSubview(label)
    }
    
    //Cài đặt máy ảnh
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        self.loadConfigUI()
        self.lbDescription.text = self.descriptionScreen
        captureSession.sessionPreset = .high
        
        guard let backCamera = AVCaptureDevice.default(for: AVMediaType.video)
            else {
                print("Unable to access back camera!")
                return
        }
        
        do {
            let input = try AVCaptureDeviceInput(device: backCamera)

            if captureSession.canAddInput(input) {
                captureSession.addInput(input)
            }
            
            if captureSession.canAddOutput(stillImageOutput) {
                captureSession.addOutput(stillImageOutput)
                setupLivePreview()
            }
        }
        catch let error  {
            print("Error Unable to initialize back camera:  \(error.localizedDescription)")
        }
        
        self.getCameraFrames()
        
        DispatchQueue.global(qos: .userInitiated).async {
            self.captureSession.startRunning()
        }
        
        DispatchQueue.main.async {
            self.videoPreviewLayer.frame = self.viewCamera.bounds
        }
    }
    
    
    //set frame preview
    func setupLivePreview() {
        videoPreviewLayer = AVCaptureVideoPreviewLayer(session: captureSession)
        
        videoPreviewLayer.videoGravity = .resizeAspectFill
        videoPreviewLayer.connection?.videoOrientation = .portrait
        viewCamera.layer.addSublayer(videoPreviewLayer)
        
        if let viewWithTag = self.viewCamera.viewWithTag(11) {
            viewWithTag.removeFromSuperview()
        }
        
        if let viewWithTag = self.viewCamera.viewWithTag(22) {
            viewWithTag.removeFromSuperview()
        }
        
        let overlay = createOverlay(frame: view.frame, xOffset: view.frame.midX, yOffset: view.frame.midY, radius: 50.0)
        overlay.tag = 11
        viewCamera.addSubview(overlay)
    }
    
    //set vùng đặt thẻ
    func createOverlay(frame: CGRect, xOffset: CGFloat, yOffset: CGFloat, radius: CGFloat, colorBorder: CGColor = UIColor.clear.cgColor) -> UIView {
        let overlayView = UIView(frame: frame)
        overlayView.backgroundColor = UIColor.black.withAlphaComponent(0.6)
        let path = CGMutablePath()
        
        let fWidth = self.viewCamera.frame.size.width
        let fHeight = self.viewCamera.frame.size.height
        let leftSpace = fWidth/2 - (fHeight * 8 / 10 / 2 * 3 / 4)
        
        if self.statusScreen == "horizontal" {
            path.addRoundedRect(in: CGRect(x: leftSpace, y: self.viewCamera.frame.origin.y,
             width: fHeight * 8 / 10 * 3 / 4, height: fHeight * 8 / 10 ),
            cornerWidth: 20, cornerHeight: 20)
        } else {
            path.addRoundedRect(in: CGRect(x: fWidth/20, y: fHeight / 8,
             width: fWidth * 18 / 20, height: fWidth * 18 * 3 / 20 / 4 ),
            cornerWidth: 20, cornerHeight: 20)
            
        }
        
        let borderLayer = CAShapeLayer()
        borderLayer.path = path // Reuse the Bezier path
        borderLayer.fillColor = UIColor.clear.cgColor
        borderLayer.strokeColor = colorBorder
        borderLayer.lineWidth = 5
        borderLayer.frame = overlayView.bounds
        overlayView.layer.addSublayer(borderLayer)

        path.addRect(CGRect(origin: .zero, size: overlayView.frame.size))
        
        let maskLayer = CAShapeLayer()
        maskLayer.backgroundColor = UIColor.black.cgColor
        maskLayer.path = path
        maskLayer.fillRule = .evenOdd
        overlayView.layer.mask = maskLayer
        overlayView.clipsToBounds = true
        
        return overlayView
    }

    //Xử lý data sâu khi chụp
    @available(iOS 11.0, *)
    func photoOutput(_ output: AVCapturePhotoOutput, didFinishProcessingPhoto photo: AVCapturePhoto, error: Error?) {
        
        guard let imageData = photo.fileDataRepresentation()
            else { return }
        
        self.captureSession.stopRunning()
        self.captureSession.removeOutput(self.videoDataOutput)
        self.captureSession.removeOutput(self.stillImageOutput)
        
        let viewController = SBKResultCapture()
        viewController.imageData = imageData
        viewController.checkScreen = self.checkScreen
        viewController.idFront = self.idFront
        viewController.URLToken = self.URLToken
        viewController.statusScreen = self.statusScreen
        viewController.completionSuccessResultCard = { [weak self] data in
            guard let `self` = self else { return }
            self.completionSuccessCard(data)
        }
        
        self.navigationController?.pushViewController(viewController, animated: true)
    }
    
    //Sự kiện chụp ảnh
    @IBAction func onCapturePhoto(_ sender: Any) {
        if (self.statusValidateImage == ValidateCard.IMAGEFRONT && self.checkScreen == 1) || (self.statusValidateImage == .IMAGEBACK && self.checkScreen == 2) {
            if #available(iOS 11.0, *) {
                let settings = AVCapturePhotoSettings(format: [AVVideoCodecKey: AVVideoCodecType.jpeg])
                stillImageOutput.capturePhoto(with: settings, delegate: self)
            } else {
                let settings = AVCapturePhotoSettings(format: [AVVideoCodecKey: AVVideoCodecJPEG])
                stillImageOutput.capturePhoto(with: settings, delegate: self)
            }
        }
    }
    
    func resizeImageCard(pixelBuffer: CVPixelBuffer) -> CVPixelBuffer {
        let validateView = SBKValidateInput.shared
        let imageInput = validateView.convertCVPixelToUIImage(pixelBuffer: pixelBuffer)
        var imageOutput: UIImage?
        if self.statusScreen == "horizontal" {
            imageOutput = validateView.cropImageHorizontal(image: imageInput, rect: CGRect(x: imageInput.size.width * 1 / 10, y: imageInput.size.height * 3 / 20, width: imageInput.size.width * 8 / 10, height: imageInput.size.height * 9 / 10), scale: 1.0)
        } else {
            imageOutput = validateView.cropImage(image: imageInput, rect: CGRect(x: imageInput.size.width / 20, y: imageInput.size.height / 8, width: imageInput.size.width * 18 / 20, height: imageInput.size.width * 18 / 20 * 3 / 4), scale: 1.0)
        }
        
        let ciimage = CIImage(image: imageOutput!)
        let tmpcontext = CIContext(options: nil)
        let cgimage =  tmpcontext.createCGImage(ciimage!, from: ciimage!.extent)
        
        return validateView.convertCGImgeToCVPixelBuffer(forImage: cgimage!)!
    }
    
    @IBAction func onSelectRotateScreen(_ sender: Any) {
    }
    
    @IBAction func onSelectBack(_ sender: Any) {
        if checkScreen == 1 {
            self.navigationController?.popViewController(animated: true)
        } else {
            for controller in self.navigationController!.viewControllers as Array {
                if controller.isKind(of: SBKCaptureCardVC.self) {
                    self.navigationController!.popToViewController(controller, animated: true)
                    break
                }
            }
        }
    }
    
}

extension SBKCaptureCardVC: AVCaptureVideoDataOutputSampleBufferDelegate {
    private func getCameraFrames() {
        self.videoDataOutput.videoSettings = [(kCVPixelBufferPixelFormatTypeKey as NSString) : NSNumber(value: kCVPixelFormatType_32BGRA)] as [String : Any]
        self.videoDataOutput.alwaysDiscardsLateVideoFrames = true
        self.videoDataOutput.setSampleBufferDelegate(self, queue: DispatchQueue(label: "camera_frame_processing_queue"))
        self.captureSession.addOutput(self.videoDataOutput)
        guard let connection = self.videoDataOutput.connection(with: AVMediaType.video),
            connection.isVideoOrientationSupported else { return }
        connection.videoOrientation = .portrait
    }
    
    func captureOutput( _ output: AVCaptureOutput, didOutput sampleBuffer: CMSampleBuffer, from connection: AVCaptureConnection) {
        
        guard let imageFrameInput = CMSampleBufferGetImageBuffer(sampleBuffer) else {
            debugPrint("unable to get image from sample buffer")
            return
        }
        
        let validateImageCard = validateInput.didOutput(pixelBuffer: self.resizeImageCard(pixelBuffer: imageFrameInput))
        
        DispatchQueue.main.async {
            if let viewWithTag = self.viewCamera.viewWithTag(11) {
                viewWithTag.removeFromSuperview()
            }
            self.statusValidateImage = validateImageCard
            if validateImageCard == ValidateCard.IMAGEFAKE || validateImageCard == .ERROR {
                self.lbDescription.textColor = UIColor.red
                self.lbDescription.text = "Incorrect card, please check!".localized()
                self.imgCaution.isHidden = false
                self.btnCapture.setImage(UIImage(named: "Button_Do@2x", in: Bundle(for: SBKTutorialVC.self), compatibleWith: nil), for: .normal)
                let overlay = self.createOverlay(frame: self.view.frame, xOffset: self.view.frame.midX, yOffset: self.view.frame.midY, radius: 50.0, colorBorder: UIColor.red.cgColor)
                overlay.tag = 11
                self.viewCamera.addSubview(overlay)
            }
            
            if validateImageCard == .IMAGEFRONT && self.checkScreen == 2 {
                self.lbDescription.textColor = UIColor.red
                self.lbDescription.text = "Please put the back of the card in".localized()
                self.btnCapture.setImage(UIImage(named: "Button_Do@2x", in: Bundle(for: SBKTutorialVC.self), compatibleWith: nil), for: .normal)
                self.imgCaution.isHidden = false
                let overlay = self.createOverlay(frame: self.view.frame, xOffset: self.view.frame.midX, yOffset: self.view.frame.midY, radius: 50.0, colorBorder: UIColor.red.cgColor)
                overlay.tag = 11
                self.viewCamera.addSubview(overlay)
            }
            
            if validateImageCard == .IMAGEBACK && self.checkScreen == 1 {
                self.lbDescription.textColor = UIColor.red
                self.lbDescription.text = "Please put the front of the card in".localized()
                self.btnCapture.setImage(UIImage(named: "Button_Do@2x", in: Bundle(for: SBKTutorialVC.self), compatibleWith: nil), for: .normal)
                self.imgCaution.isHidden = false
                let overlay = self.createOverlay(frame: self.view.frame, xOffset: self.view.frame.midX, yOffset: self.view.frame.midY, radius: 50.0, colorBorder: UIColor.red.cgColor)
                overlay.tag = 11
                self.viewCamera.addSubview(overlay)
            }
            
            if (self.statusValidateImage == ValidateCard.IMAGEFRONT && self.checkScreen == 1) || (self.statusValidateImage == .IMAGEBACK && self.checkScreen == 2) {
                self.lbDescription.textColor = UIColor.colorFromHexa(Global.colorConfig)
                self.lbDescription.text = "Are you ready. Let's start!".localized()
                self.imgCaution.isHidden = true
                self.btnCapture.setImage(UIImage(named: "iconCap", in: Bundle(for: SBKTutorialVC.self), compatibleWith: nil), for: .normal)
                let overlay = self.createOverlay(frame: self.view.frame, xOffset: self.view.frame.midX, yOffset: self.view.frame.midY, radius: 50.0, colorBorder: UIColor.blue.cgColor)
                overlay.tag = 11
                self.viewCamera.addSubview(overlay)
            }
        }
    }
}
